/* 
    buz - Iomega Buz frame grabber driver

    Copyright (C) 1998 Dave Perks <dperks@ibm.net>

	based on

    bttv - Bt848 frame grabber driver

    Copyright (C) 1996,97 Ralph Metzler (rjkm@thp.uni-koeln.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _BUZ_H_
#define _BUZ_H_

#define	BUZ_MAX_WIDTH	720		/* never display more than 720 pixels */
#define	BUZ_MAX_HEIGHT	576		/* never display more than 576 rows */
#define	BUZ_MIN_WIDTH	32		/* never display less than 32 pixels */
#define	BUZ_MIN_HEIGHT	32		/* never display less than 32 rows */

#define MAX_CLIPRECS	100

#define BUZ_NUM_STAT_COM	4
#define BUZ_MASK_STAT_COM	3

#if VIDEO_MAX_FRAME <= 32
#	define	BUZ_MAX_FRAME	32
#elif VIDEO_MAX_FRAME <= 64
#	define	BUZ_MAX_FRAME	64
#else
#	error	"Too many video frame buffers to handle"
#endif
#define	BUZ_MASK_FRAME	(BUZ_MAX_FRAME - 1)

struct buz_codec_control {
	int	quality;	/* desired JPEG code size in bytes */
	char	comment[62];	/* contents of COM marker, NUL terminated */
};

struct buz_mmap
{
	unsigned	int frame;		/* Frame (0 - n) for double buffer */
	int		height,width;
	unsigned	int format;		/* should be VIDEO_PALETTE_* */
	unsigned	int length;		/* number of code bytes in buffer */
};

struct buz_sync
{
	unsigned	int frame;		/* Frame (0 - n) for double buffer */
	unsigned	int length;		/* number of code bytes in buffer */
	unsigned	int seq;		/* frame sequence number */
	struct timeval	timestamp;		/* timestamp */
};

/*
Private IOCTL to set up for displaying MJPEG
*/
#define BUZIOCMPLAY		_IOW('v', BASE_VIDIOCPRIVATE+0,  struct buz_mmap)
#define BUZIOCSYNC		_IOWR('v', BASE_VIDIOCPRIVATE+1,  struct buz_sync)
#define BUZIOCSCODEC		_IOW('v', BASE_VIDIOCPRIVATE+2,  struct buz_codec_control)


#ifdef __KERNEL__

#include "zr36057.h"

#define	BUZ_NUM_FRAGS	(BUZ_MAX_FBUF / PAGE_SIZE + 2)

struct buz_window 
{
	ushort bpp;
	int norm;
	int color_fmt;
};

enum buz_codec_mode
{
	BUZ_MODE_IDLE,				/* nothing going on */
	BUZ_MODE_MOTION_COMPRESS,		/* grabbing frames */
	BUZ_MODE_MOTION_DECOMPRESS,		/* playing frames */
	BUZ_MODE_STILL_COMPRESS,		/* still frame conversion */
	BUZ_MODE_STILL_DECOMPRESS		/* still frame conversion */
};

struct buz_codec
{
	enum buz_codec_mode mode;		/* current transfer mode */
	ushort quality;				/* quality setting, 0..65535 */
	char	comment[62];			/* contents of COM marker, NUL terminated */
};

enum buz_buffer_state
{
	BUZ_STATE_USER,				/* buffer is owned by application */
	BUZ_STATE_PEND,				/* buffer is queued in pend[] ready to feed to I/O */
	BUZ_STATE_DMA,				/* buffer is queued in dma[] for I/O */
	BUZ_STATE_DONE				/* buffer is ready to return to application */
};

struct buz_gbuffer
{
	char * fbuffer;				/* addresses of frame buffer */
	u32 * frag_tab;				/* addresses of frag table */
	u32 frag_tab_bus;			/* same value cached to save time in ISR */
	enum buz_buffer_state state;		/* non-zero if corresponding buffer is in use in grab queue */
	int stat_com_index;			/* index into stat_com[] */
	struct wait_queue * capq;		/* wait here for grab to finish */
	int result;				/* DONE: result to return to application */
	struct buz_sync bs;			/* DONE: info to return to application */
};

struct buz 
{
	struct video_device video_dev;
	struct i2c_bus i2c;

	struct video_picture picture;		/* Current picture params */
	struct video_buffer buffer;		/* Current buffer params */
	struct video_window window;		/* Current window params */
	struct video_capture capture;		/* Current capture params */
	struct buz_window win;
	struct buz_codec codec;
	u32 *overlay_mask;

	unsigned short id;
	char name[32];			/* name of this device */
	struct pci_dev *pci_dev;          /* PCI device */
	unsigned char revision;		/* revision of zr36057 */
	unsigned int zr36057_adr;      /* bus address of IO mem returned by PCI BIOS */
	unsigned char *zr36057_mem;   /* pointer to mapped IO memory */

	int need_contiguous;		/* non-zero if rev 1 chip needs contiguous frame buffers */
	int max_fbuf;			/* maximum frame buffer size */
  
	int user;

	int cap;

	u16 gwidth, gheight, gfmt;
	int gmode;

	/* this spinlock protects all variables below this line */
	spinlock_t lock;

	/* grab queue counts/indices, mask with BUZ_MASK_STAT_COM before using as index */
	/* (dma_tail - dma_head) is number active in DMA, must be <= BUZ_NUM_STAT_COM */
	/* (value & BUZ_MASK_STAT_COM) corresponds to index in stat_com table */
	unsigned dma_head;			/* first grab buffer in active queue */
	unsigned dma_tail;			/* first grab buffer after queue */
	unsigned long grab_seq;			/* count of frames since grab started */
	unsigned long grab_lost;		/* count of frames dropped during grab */

	/* zr36057's code buffer table */
	u32 * stat_com;				/* stat_com[i] is indexed by dma_head/tail & BUZ_MASK_STAT_COM */
	int dma[BUZ_NUM_STAT_COM]; 		/* this too, indicates which frag buffer is here */

	/* queue containing indexes of frames in BUZ_STATE_PEND state */
	/* (pend_tail - pend_head) is number pending, must be <= VIDEO_MAX_FRAME */
	/* (value & BUZ_MASK_FRAME) corresponds to index in pend[] queue */
	unsigned pend_head;
	unsigned pend_tail;
	int pend[BUZ_MAX_FRAME];

	/* array indexed by frame number */
	struct buz_gbuffer gbuf[VIDEO_MAX_FRAME]; /* frame buffers' info */
};

#endif

/*The following should be done in more portable way. It depends on define
  of _ALPHA_BUZ in the Makefile.*/

#ifdef _ALPHA_BUZ
#define btwrite(dat,adr)    writel((dat),(char *) (btv->zr36057_adr+(adr)))
#define btread(adr)         readl(btv->zr36057_adr+(adr))
#else
#define btwrite(dat,adr)    writel((dat), (char *) (btv->zr36057_mem+(adr)))
#define btread(adr)         readl(btv->zr36057_mem+(adr))
#endif

#define btand(dat,adr)      btwrite((dat) & btread(adr), adr)
#define btor(dat,adr)       btwrite((dat) | btread(adr), adr)
#define btaor(dat,mask,adr) btwrite((dat) | ((mask) & btread(adr)), adr)

#define I2C_TSA5522        0xc2
#define I2C_TDA9850        0xb6
#define I2C_HAUPEE         0xa0
#define I2C_STBEE          0xae
#define	I2C_SAA7111        0x48
#define	I2C_SAA7185        0x88

#define TDA9850_CON1       0x04
#define TDA9850_CON2       0x05
#define TDA9850_CON3       0x06
#define TDA9850_CON4       0x07
#define TDA9850_ALI1       0x08
#define TDA9850_ALI2       0x09
#define TDA9850_ALI3       0x0a

#endif
